@@ -11,6 +11,8 @@ Features:
   - Integrated selenium support:
     - The test runner transparently starts its own self-contained (CherryPy)
       server that the selenium tests access.
+    - We have a "--max-tries" argument that you can use to retry failed tests,
+      since selenium tests can be fragile and rely on timing.
   - Support for separate workers if you want to run tests in parallel;
     vastly improves test suite run times for selenium tests.
   - Can run tests by partial paths rather than full paths (eg. can specify
@@ -85,8 +87,10 @@ PLUGINS = [import_object(path)() for path in settings.TEST_PLUGINS]
 command_line_options = [
   OptionDescriptor("-g", "--grep", dest = "grep", action = "append"),
   OptionDescriptor("-t", "--tag", dest = "tag", action = "store"),
-  OptionDescriptor("-m", "--max_tries", dest = "max_tries", action = "store", 
+  OptionDescriptor("-m", "--max-tries", dest = "max_tries", action = "store", 
                          type = "int", default = 1),
+  OptionDescriptor("-n", "--never-debug", dest = "never_debug", action = "store_true", 
+                         default = False),
   #TODO:
   #make_option('--numworkers', '-n', dest='num_workers',
   #    help='The number of workers.'),
@@ -98,8 +102,8 @@ for p in PLUGINS:
   command_line_options.extend(p.get_command_line_options())
   
 
-def for_all_plugins(self, method_name, *args, **kwargs):
-  for p in self.plugins:
+def for_all_plugins(method_name, *args, **kwargs):
+  for p in PLUGINS:
     getattr(p, method_name)(*args, **kwargs)
 
 
@@ -268,7 +272,7 @@ class TestDriver(object):
           st = traceback.format_tb(tb)
           ex = traceback.format_exception_only(etype, value)
           
-          for_all_plugins("test_failure", self, klass, method_name, ex, tb)
+          for_all_plugins("test_failure", self, klass, method_name, ex, st)
           
           for line in itertools.chain(st, ex):
             print re.sub('(' + os.getcwd() + ')(.*)", (line \\d+)', 
@@ -341,6 +345,8 @@ class CustomTestRunner(object):
   def run_tests(self, *test_paths, **options):
     driver = TestDriver(max_tries = options['max_tries'])
 
+    driver.never_debug = options["never_debug"]
+
     if test_paths:
       for p in test_paths:
         driver.add_path(p)
@@ -353,7 +359,7 @@ class CustomTestRunner(object):
       driver.add_filter(lambda _1, method, _: options['tag'] in getattr(method, 'tags', []))
     else:
       add_all_tests = True
-      for p in self.plugins:
+      for p in PLUGINS:
         handled = p.process_command_line_options(driver, **options)
         if handled:
           add_all_tests = False
